# !SetUpBookStackERC.py
# Script for setting the limits, initial values, visibility, etc for parameters in all the books in a bookstack figure (or stack of such)

import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
# From Sep 2024 I've added the 'main' and 'del' stuff at the bottom as per D3Ds 'UnHide Dials' script to tidy things up after running the script
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures to which this, or any of its parent, BookStack4 figures are parented
	def getAnyParentStackFigures(self,thisfig):
		act = thisfig.ParentActor()
		parentact = act.Parent()
		parentactname = parentact.Name()
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigures(self, thisfig):
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyChildStackFigures(childfig)
					
		return
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the user gets cold feet right at the start
		confirmed = poser.DialogSimple.YesNo("FOR SETTING UP THE FINAL ERC FOR A BOOKSTACK\n\nDo you wish to continue?")
		if not confirmed:
			return
			
		# Abort if the selected figure's not a BookStack4
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		#parentstoo=poser.DialogSimple.YesNo("Parent bookstack figures too?")
		#if parentstoo:
			#self.getAnyParentStackFigures(fig)
			#self.figlist.reverse()
		
		# ...then add the current figure to the list
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		#kidstoo=poser.DialogSimple.YesNo("Child bookstack figures too?")
		#if kidstoo:
			#self.getAnyChildStackFigures(fig)
				
		bkthisfig = 0 # number of books in this figure (obtained from figure name)
		for f in self.figlist:
		
			# Get the figure
			fig = poser.Scene().Figure(f)
			
			# Print figure and actor names
			print ("\nList of actors in figure: : " + fig.Name())
			actlist = fig.Actors()
			for a in actlist:
				print ("  - " + a.Name() + " (" + a.InternalName() + ")")
			
			# Extract the number of books in the figure from the name, e.g. "BookStack4 - books 1-4", "BookStack8 - books 1-8", "BookStack32 - books 1-32"
			# (This is in case I ever create the larger BookStack figures)
			if fig.Name()[9].isdigit():
				if fig.Name()[10].isdigit():
					bkthisfig = int(fig.Name()[9])*10 + int(fig.Name()[10])
				else:
					bkthisfig = int(fig.Name()[9])
		
			print("\n")

			# The intermediate math valueParms already have ERC since I cut-paste-edit that block
			# valueOpDeltaAddDelta - although valueOpPlus is better if deltaAddDelta=1
			# (for DeltaAddDelta an extra deltaAddDelta value is required - if it'll be set to 1 use valueOpPlus instead)
			voppluslist = (("MT PB# Curl","Curl (Paperback only)"),
			("MT PB# Plump","Plump"),
			("MT PB# Shear","Shear"),
			("MT PB# Twist","Twist"),
			("MT HBf# Plump","Plump"),
			("MT HBf# Shear","Shear"),
			("MT HBf# Twist","Twist"),
			("MT HBc# Plump","Plump"),
			("MT HBc# Shear","Shear"),
			("MT HBc# Twist","Twist"),
			("yRotate","Rotate"),
			("zRotate","Sx3xcosYR+Ox2xsinYR"),
			("xRotate","Sx3xsinYR+Ox2xcosYR"))
			
			# valuOpTimes
			voptimeslist = (("Scale","Book size"),
			("xScale","Cover proportions"),
			("yScale","Book thickness"),
			("xScale","Cover proportions"))
			
			# valueOpDeltaAdd - third parameter in each tuple is the (non-unity) delta
			vopdadlist = (("yScale","Splay",0.12),
			("xTran","Nudge sideways",0.0025),
			("zTran","Nudge lengthwise",0.005))
			
			# For each of the books in the figure
			i = 1
			while i < (bkthisfig+1):
				
				# Get the relevant bone/bodypart
				bone = "book_" + str(i)
				print("\n"+bone+"\n")
				act = fig.ActorByInternalName(bone)
				
				if i > 8:
					for v in voppluslist:
						t = v[0].replace("#",str(i))
						print(" - Slaving '"+t+"' to '"+v[1]+"' with a valueOpPlus")
						act.Parameter(t).AddValueOperation(poser.kValueOpTypeCodePLUS,act.Parameter(v[1]))
					for v in voptimeslist:
						print(" - Slaving '"+v[0]+"' to '"+v[1]+"' with a valueOpTimes")
						act.Parameter(v[0]).AddValueOperation(poser.kValueOpTypeCodeTIMES,act.Parameter(v[1]))
					for v in vopdadlist:
						print(" - Slaving '"+v[0]+"' to '"+v[1]+"' with a valueOpDeltaAdd...")
						act.Parameter(v[0]).AddValueOperation(poser.kValueOpTypeCodeDELTAADD,act.Parameter(v[1]))
						oplist = act.Parameter(v[0]).ValueOperations()
						for o in oplist:
							if o.SourceParameter().Name() == v[1]:
								print("    ...and setting deltaAddDelta to "+str(v[2]))
								o.SetDelta(v[2])
					
					# add an AddDelta value op to drive yTran at 0.002326 from 'Plump' of PREVIOUS bone
					prevbone = "book_" + str(i-1)
					dad = 0.002326
					driver = fig.ActorByInternalName(prevbone).Parameter("Plump")
					print("\n - Slaving 'yTran' to '"+driver.Name()+"' OF PREVIOUS BONE with a valueOpDeltaAdd...")
					act.Parameter("yTran").AddValueOperation(poser.kValueOpTypeCodeDELTAADD,driver)
					oplist2 = act.Parameter("yTran").ValueOperations()
					for o in oplist2:
						if o.SourceParameter().Name() == "Plump":
							print("    ...and setting deltaAddDelta to "+str(dad))
							o.SetDelta(dad)
				
				# Increment book number for next iteration
				i = i + 1
			
		# Redraw what we've modified	
		poser.Scene().Draw()


def main():
	poser.DialogSimple.MessageBox("Running 3DCheapskates '!SetUpBookStackERC.py' script\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
	cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
	cheapskatesTemporaryInstance.main()
	poser.DialogSimple.MessageBox("3DCheapskates '!SetUpBookStackERC.py' script all done.")

main()

del main
del cheapskatesTemporaryClass